// SPDX-License-Identifier: Unlicense

// Package test contains helpers for testing parsers.
package test

import (
	"errors"
	"io"

	"git.codemonkeysoftware.net/b/gigaparsec"
	"github.com/shoenig/test"
)

type errReaderAt struct {
	err error
}

func (r errReaderAt) ReadAt([]byte, int64) (int, error) {
	return 0, r.err
}

// ErrReaderAt returns an [io.ReaderAt] with a ReadAt method that always returns err.
func ErrReaderAt(err error) io.ReaderAt {
	return errReaderAt{err: err}
}

func StateIsAt[Input any](t test.T, s gigaparsec.State[Input], pos uint64) {
	test.EqOp(t, pos, s.Pos(), test.Sprintf("expected parser state to be at position %d, got %d", pos, s.Pos()))
}

func SliceOfN[T any](value T, n int) []T {
	s := make([]T, n)
	for i := range s {
		s[i] = value
	}
	return s
}

type ForcedResult struct{ Succeed, Consume bool }

func ForceResult(state gigaparsec.State[ForcedResult]) (gigaparsec.Result[ForcedResult, struct{}], error) {
	buf := make([]ForcedResult, 1)
	_, next, err := state.Read(buf)
	if errors.Is(err, io.EOF) {
		return gigaparsec.Fail[ForcedResult, struct{}](false, gigaparsec.MessageEnd(state.Pos())), nil
	}
	if err != nil {
		return gigaparsec.Result[ForcedResult, struct{}]{}, err
	}
	tok := buf[0]
	if tok.Succeed {
		return gigaparsec.Succeed(tok.Consume, struct{}{}, next, gigaparsec.MessageOK(state.Pos())), nil
	} else {
		return gigaparsec.Fail[ForcedResult, struct{}](tok.Consume, gigaparsec.MakeMessage(state.Pos(), "Succeed=false", "Succeed=true")), nil
	}
}
